@@ -58,7 +58,6 @@ import asyncio
 import concurrent.futures
 import logging
 import multiprocessing
-import queue
 import threading
 import warnings
 from abc import ABCMeta, abstractmethod
@@ -74,7 +73,7 @@ from overrides import EnforceOverrides, overrides
 from .remote_exception import RemoteException, exit_err_msg
 from .util import forward_logs, logger_thread
 from .streamer import put_in_queue
-from . import mpqueue  # noqa: F401
+from . import _queues
 
 
 # Set level for logs produced by the standard `multiprocessing` module.
@@ -83,20 +82,8 @@ multiprocessing.log_to_stderr(logging.WARNING)
 logger = logging.getLogger(__name__)
 
 
-class EnqueueTimeout(Exception):
-    def __init__(self, duration, queue_idx):
-        super().__init__(duration, queue_idx)
-
-    def __str__(self):
-        return f"enqueue timed out after {self.args[0]} seconds at queue {self.args[1]}"
-
-
-class TotalTimeout(Exception):
-    def __init__(self, duration):
-        super().__init__(duration)
-
-    def __str__(self):
-        return f"total timed out after {self.args[0]} seconds"
+class TimeoutError(Exception):
+    pass
 
 
 class Servlet(metaclass=ABCMeta):
@@ -105,8 +92,8 @@ class Servlet(metaclass=ABCMeta):
 
     @classmethod
     def run(cls, *,
-            q_in: queues.Queue,
-            q_out: queues.Queue,
+            q_in: _queues.Unique,
+            q_out: _queues.Unique,
             q_err: queues.Queue,
             q_log: queues.Queue,
             should_stop: synchronize.Event,
@@ -116,14 +103,18 @@ class Servlet(metaclass=ABCMeta):
         This classmethod runs in the worker process to construct
         the worker object and start its processing loop.
 
-        This function is the parameter `target` to `multiprocessing.Process`
+        This function is the parameter `target` to `Process`
         called by `MPServer` in the main process. As such, elements
         in `init_kwargs` go through pickling, hence they should consist
         mainly of small, native types such as string, number,small dict.
         Be careful about passing custom class objects in `init_kwargs`.
         '''
+        batch_size = max(1, init_kwargs.get('batch_size') or 1)
+        q_in = q_in.reader(batch_size + 10, batch_size)
+        q_out = q_out.writer()
         try:
             forward_logs(q_log)
+            # TODO: does the log message carry process info?
             if cpus:
                 psutil.Process().cpu_affinity(cpus=cpus)
             obj = cls(**init_kwargs)
@@ -193,11 +184,14 @@ class Servlet(metaclass=ABCMeta):
 
     def start(self, *, q_in, q_out, q_err, should_stop):
         q_err.put(self.name)
-        if self.batch_size > 1:
-            self._start_batch(q_in=q_in, q_out=q_out, q_err=q_err,
-                              should_stop=should_stop)
-        else:
-            self._start_single(q_in=q_in, q_out=q_out, q_err=q_err, should_stop=should_stop)
+        try:
+            if self.batch_size > 1:
+                self._start_batch(q_in=q_in, q_out=q_out, q_err=q_err,
+                                  should_stop=should_stop)
+            else:
+                self._start_single(q_in=q_in, q_out=q_out, q_err=q_err, should_stop=should_stop)
+        except KeyboardInterrupt:
+            print(self.name, 'stopped by KeyboardInterrupt')
 
     def _start_single(self, *, q_in, q_out, q_err, should_stop):
         batch_size = self.batch_size
@@ -222,8 +216,9 @@ class Servlet(metaclass=ABCMeta):
                 # and details later using the `RemoteException`
                 # object. Be brief on the logging here.
                 err = RemoteException().to_dict()
-                if not put_in_queue(q_err, (uid, err), should_stop):
-                    return
+                q_err.put((uid, err))
+            if should_stop.is_set():
+                break
 
     def _start_batch(self, *, q_in, q_out, q_err, should_stop):
         batch_size = self.batch_size
@@ -234,48 +229,60 @@ class Servlet(metaclass=ABCMeta):
                         n_batches, batch_size_min, batch_size_max, batch_size_mean)
 
         n_batches = 0
-        while True:
-            if n_batches == 0:
-                batch_size_max = -1
-                batch_size_min = 1000000
-                batch_size_mean = 0.0
+        try:
+            while True:
+                if n_batches == 0:
+                    batch_size_max = -1
+                    batch_size_min = 1000000
+                    batch_size_mean = 0.0
+
+                try:
+                    batch = q_in.get_many(batch_size, first_timeout=0.5, extra_timeout=batch_wait_time)
+                except Empty:
+                    if should_stop.is_set():
+                        if n_batches:
+                            print_batching_info()
+                        return
+                    continue
+                except Exception as e:
+                    print(type(e), repr(e), str(e))
+                    raise
+
+                uids = [v[0] for v in batch]
+                batch = [v[1] for v in batch]
+                n = len(batch)
+
+                try:
+                    results = self.call(batch)
+                except Exception:
+                    err = RemoteException().to_dict()
+                    for uid in uids:
+                        q_err.put((uid, err))
+                else:
+                    while True:
+                        try:
+                            q_out.put_many(zip(uids, results), timeout=0.1)
+                            break
+                        except Full:
+                            if should_stop.is_set():
+                                return
+
+                n_batches += 1
+                batch_size_max = max(batch_size_max, n)
+                batch_size_min = min(batch_size_min, n)
+                batch_size_mean = (batch_size_mean * (n_batches - 1) + n) / n_batches
+                if n_batches % 1000 == 0:
+                    print_batching_info()
+                    n_batches = 0
 
-            try:
-                batch = q_in.get_many(batch_size, first_timeout=0.5, extra_timeout=batch_wait_time)
-            except Empty:
                 if should_stop.is_set():
-                    if n_batches:
+                    if n_batches and n_batches % 1000 != 0:
                         print_batching_info()
                     return
-                continue
-            except Exception as e:
-                print(type(e), repr(e), str(e))
-                raise
-
-            uids = [v[0] for v in batch]
-            batch = [v[1] for v in batch]
-            n = len(batch)
-
-            try:
-                results = self.call(batch)
-            except Exception:
-                err = RemoteException().to_dict()
-                q_err.put_many([(uid, err) for uid in uids])
-            else:
-                q_out.put_many(list(zip(uids, results)))
-
-            n_batches += 1
-            batch_size_max = max(batch_size_max, n)
-            batch_size_min = min(batch_size_min, n)
-            batch_size_mean = (batch_size_mean * (n_batches - 1) + n) / n_batches
-            if n_batches % 1000 == 0:
+        except BaseException:
+            if n_batches:
                 print_batching_info()
-                n_batches = 0
-
-            if should_stop.is_set():
-                if n_batches and n_batches % 1000 != 0:
-                    print_batching_info()
-                return
+            raise
 
     @abstractmethod
     def call(self, x):
@@ -317,24 +324,29 @@ class Servlet(metaclass=ABCMeta):
 
 
 class MPServer(EnforceOverrides, metaclass=ABCMeta):
-    SLEEP_ENQUEUE = 0.00015
-    # Sleep when queue is full. It is not disastrous if this is a little
-    # long, because this happens only occasionally.
+    '''
+    In previous versions, the input queue has a configurable maxsize to regulate
+    the number of in-progress items in the pipeline.
+    Since version 0.10.6, the input queue has effectively unlimitted capacity.
+    Input load is regulated in two ways:
+
+        - When this object is behind a HTTP service, the server's `backlog`
+          parameter provides this regulation.
+        - When this object is used via its method `stream`, the `backlog`
+          parameter of `stream` provides this regulation.
+    '''
     SLEEP_DEQUEUE = 0.00004
     SLEEP_FUTURE = SLEEP_DEQUEUE
     # Sleep when future is not done yet or queue is empty.
     # This may better be short, because we want to go once things are available.
-    TIMEOUT_ENQUEUE = 1
-    TIMEOUT_TOTAL = 10
 
-    def __init__(self, *, main_cpu: int = 0, queue_type: str = 'BasicQueue'):
+    def __init__(self, *, main_cpu: int = 0):
         '''
         `main_cpu`: specifies the cpu for the "main process",
         i.e. the process in which this server objects resides.
         '''
-        self._queue_type = queue_type
-        self._q_err = self.get_mpcontext().BasicQueue()
-        self._q_log = self.get_mpcontext().BasicQueue()  # This does not need the performance optim of fast_fifo.
+        self._q_err = self.get_mpcontext().Queue()
+        self._q_log = self.get_mpcontext().Queue()  # This does not need the performance optim of fast_fifo.
         self._servlet_configs = []
         self._servlets: List[multiprocessing.Process] = []
         self._uid_to_futures = {}
@@ -369,18 +381,20 @@ class MPServer(EnforceOverrides, metaclass=ABCMeta):
         # should be used with context manager `__enter__`/`__exit__`
 
         t = self._thread_pool.submit(logger_thread, self._q_log)
-        t.add_done_callback(self._thread_task_done_callback)
         self._thread_tasks[t] = None
+        t.add_done_callback(self._thread_task_done_callback)
+        # Put in the dict before adding callback, in case
+        # the task is already done when adding the callback.
 
         self._start_servlets()
         assert self._servlets
 
         t = self._thread_pool.submit(self._gather_output)
-        t.add_done_callback(self._thread_task_done_callback)
         self._thread_tasks[t] = None
+        t.add_done_callback(self._thread_task_done_callback)
         t = self._thread_pool.submit(self._gather_error)
-        t.add_done_callback(self._thread_task_done_callback)
         self._thread_tasks[t] = None
+        t.add_done_callback(self._thread_task_done_callback)
 
         return self
 
@@ -407,44 +421,14 @@ class MPServer(EnforceOverrides, metaclass=ABCMeta):
         # Reset CPU affinity.
         psutil.Process().cpu_affinity(cpus=[])
 
-    def _start_servlets(self):
-        # Subclass needs to add code to create
-        # the servlet processes before executing
-        # the code below.
-
-        # Start the servlets one by one.
-        n = len(self._servlets)
-        k = 0
-        for m in self._servlets:
-            logger.debug(f"starting servlet in Process {m}")
-            m.start()
-            name = self._q_err.get()
-            k += 1
-            logger.info(f"{k}/{n}: servlet <{name}> is ready")
-
-    def _stop_servlets(self):
-        for m in self._servlets:
-            m.join()
-
     def add_servlet(self, servlet: Type[Servlet], **kwargs):
         self._servlet_configs.append((servlet, kwargs))
 
-    async def async_call(self, x, /, *,
-                         enqueue_timeout: Union[int, float] = None,
-                         total_timeout: Union[int, float] = None,
-                         ):
-        enqueue_timeout, total_timeout = self._resolve_timeout(
-            enqueue_timeout=enqueue_timeout, total_timeout=total_timeout,
-        )
-        uid, fut = await self._async_call_enqueue(x, enqueue_timeout=enqueue_timeout)
-        # This could raise EnqueueTimeout.
-        return await self._async_call_wait_for_result(uid, fut, total_timeout=total_timeout)
-        # This could raise TotalTimeout or RemoteException.
+    async def async_call(self, x, /, *, timeout: Union[int, float] = 60):
+        fut = self._enqueue(x)
+        return await self._async_call_wait_for_result(fut, timeout=timeout)
 
-    def call(self, x, /, *,
-             enqueue_timeout: Union[int, float] = None,
-             total_timeout: Union[int, float] = None,
-             ):
+    def call(self, x, /, *, timeout: Union[int, float] = 60):
         '''
         Sometimes a subclass may want to override this method
         to add preprocessing of the input and postprocessing of
@@ -452,21 +436,14 @@ class MPServer(EnforceOverrides, metaclass=ABCMeta):
 
         However, be careful to maintain consistency between `call` and `stream`.
         '''
-        enqueue_timeout, total_timeout = self._resolve_timeout(
-            enqueue_timeout=enqueue_timeout, total_timeout=total_timeout,
-        )
-
-        uid, fut = self._call_enqueue(x, enqueue_timeout=enqueue_timeout)
-        # This could raise EnqueueTimeout.
-        return self._call_wait_for_result(uid, fut, total_timeout=total_timeout)
-        # This could raise TotalTimeout or RemoteException.
+        fut = self._enqueue(x)
+        return self._call_wait_for_result(fut, timeout=timeout)
 
     def stream(self, data_stream, /, *,
                return_x: bool = False,
                return_exceptions: bool = False,
                backlog: int = 2048,
-               enqueue_timeout=None,
-               total_timeout=None,
+               timeout=60,
                ):
         '''
         `backlog`: lengths of queue that holds in-progress elements:
@@ -480,31 +457,28 @@ class MPServer(EnforceOverrides, metaclass=ABCMeta):
         # For streaming, "timeout" is usually not a concern.
         # The concern is overall throughput.
 
-        enqueue_timeout, total_timeout = self._resolve_timeout(
-            enqueue_timeout=enqueue_timeout or self.TIMEOUT_ENQUEUE * 10,
-            total_timeout=total_timeout or self.TIMEOUT_TOTAL * 10)
-        tasks = queue.Queue(backlog)
+        tasks = _queues.SingleLane(backlog)
         nomore = object()
 
         def _enqueue():
             # Putting input data in the queue does not need concurrency.
             # The speed of sequential push is as fast as it can go.
-            enqueue = self._call_enqueue
+            enqueue = self._enqueue
             tt = tasks
-            et = enqueue_timeout
             should_stop = self._should_stop
             for x in data_stream:
                 try:
-                    uid, fut = enqueue(x, enqueue_timeout=et)
+                    fut = enqueue(x)
+                    timedout = False
                 except Exception as e:
                     if return_exceptions:
-                        uid = None
+                        timedout = True
                         fut = Future()
                         fut.set_exception(e)
                     else:
                         logger.error("exception '%r' happened for input '%s'", e, x)
                         raise
-                if not put_in_queue(tt, (x, uid, fut), should_stop):
+                if not put_in_queue(tt, (x, fut, timedout), should_stop):
                     return
             put_in_queue(tt, nomore, should_stop)
             # Exceptions in `fut` is covered by `return_exceptions`.
@@ -513,8 +487,8 @@ class MPServer(EnforceOverrides, metaclass=ABCMeta):
             # it will be propagated in the main thread.
 
         t = self._thread_pool.submit(_enqueue)
-        t.add_done_callback(self._thread_task_done_callback)
         self._thread_tasks[t] = None
+        t.add_done_callback(self._thread_task_done_callback)
 
         _wait = self._call_wait_for_result
 
@@ -533,8 +507,8 @@ class MPServer(EnforceOverrides, metaclass=ABCMeta):
 
             if z is nomore:
                 break
-            x, uid, fut = z
-            if uid is None:
+            x, fut, timedout = z
+            if timedout:
                 # This happens only when `return_exceptions` is True.
                 if return_x:
                     yield x, fut.exception()
@@ -542,8 +516,8 @@ class MPServer(EnforceOverrides, metaclass=ABCMeta):
                     yield fut.exception()
             else:
                 try:
-                    y = _wait(uid, fut, total_timeout=total_timeout)
-                    # May raise RemoteException or TotalTimeout.
+                    y = _wait(fut, timeout=timeout)
+                    # May raise RemoteException or TimeoutError.
                 except Exception as e:
                     if return_exceptions:
                         if return_x:
@@ -565,6 +539,25 @@ class MPServer(EnforceOverrides, metaclass=ABCMeta):
         except KeyError:
             warnings.warn(f"the Future object `{t}` was already removed")
 
+    def _start_servlets(self):
+        # Subclass needs to add code to create
+        # the servlet processes before executing
+        # the code below.
+
+        # Start the servlets one by one.
+        n = len(self._servlets)
+        k = 0
+        for m in self._servlets:
+            logger.debug(f"starting servlet in Process {m}")
+            m.start()
+            name = self._q_err.get()
+            k += 1
+            logger.info(f"{k}/{n}: servlet <{name}> is ready")
+
+    def _stop_servlets(self):
+        for m in self._servlets:
+            m.join()
+
     def _add_servlet(self,
                      servlet: Type[Servlet],
                      *,
@@ -643,34 +636,6 @@ class MPServer(EnforceOverrides, metaclass=ABCMeta):
 
         return cpus
 
-    def _resolve_timeout(self, *,
-                         enqueue_timeout: float = None,
-                         total_timeout: float = None):
-        '''
-        `enqueue_timeout` is the max waittime for entering
-        the queue of this service. Unpon entering `call`
-        or `async_call`, if the data queue to workers of the service
-        (see `self._input_queues`) is full, it will wait up to
-        this long before timeout.
-
-        `total_timeout` is the max total time spent in `call`
-        or `async_call` before timeout. Upon entering `call`
-        or `async_call`, the input is placed in a queue (timeout
-        if that can't be done within `enqueue_timeout`), then
-        we'll wait for the result. If result does not come within
-        `total_timeout`, it will timeout. This waittime is counted
-        starting from the beginning of `call` or `async_call`, not
-        starting after the input has been placed in the queue.
-        '''
-        if enqueue_timeout is None:
-            enqueue_timeout = self.TIMEOUT_ENQUEUE
-        if total_timeout is None:
-            total_timeout = max(self.TIMEOUT_TOTAL, enqueue_timeout * 10)
-        if enqueue_timeout > total_timeout:
-            enqueue_timeout = total_timeout
-        return enqueue_timeout, total_timeout
-        # Accidental negative values for these are OK.
-
     @abstractmethod
     def _input_queues(self):
         raise NotImplementedError
@@ -702,7 +667,7 @@ class MPServer(EnforceOverrides, metaclass=ABCMeta):
                 fut.data['t2'] = monotonic()
             except asyncio.InvalidStateError as e:
                 if fut.cancelled():
-                    # Could have been canceled due to TotalTimeout.
+                    # Could have been canceled due to TimeoutError.
                     # logger.debug('Future object is already cancelled')
                     pass
                 else:
@@ -716,71 +681,44 @@ class MPServer(EnforceOverrides, metaclass=ABCMeta):
         fut.data = {'t0': monotonic()}
         return uid, fut
 
-    async def _async_call_enqueue(self, x, *, enqueue_timeout):
+    def _enqueue(self, x):
         uid, fut = self._enqueue_future(x)
+
+        for q in self._input_queues():
+            try:
+                # This input queue either has no size limit
+                # or has a very large capacity.
+                # If it is full right now, just time out.
+                q.put((uid, x), timeout=0)
+            except Full:
+                raise TimeoutError("input queue is full")
+
+        return fut
+
+    async def _async_call_wait_for_result(self, fut, *, timeout):
         t0 = fut.data['t0']
-
-        qs = self._input_queues()
-        t1 = t0 + enqueue_timeout
-        for iq, q in enumerate(qs):
-            while True:
-                try:
-                    q.put_nowait((uid, x))
-                    break
-                except Full:
-                    timenow = monotonic()
-                    if timenow >= t1:
-                        fut.cancel()
-                        self._uid_to_futures.pop(uid, None)
-                        raise EnqueueTimeout(timenow - t0, iq)
-                    await asyncio.sleep(min(self.SLEEP_ENQUEUE, t1 - timenow))
-
-        fut.data['t1'] = monotonic()
-        return uid, fut
-
-    async def _async_call_wait_for_result(self, uid, fut, *, total_timeout):
-        t0 = fut.data['t0']
-        t2 = t0 + total_timeout
+        t2 = t0 + timeout
         while not fut.done():
             timenow = monotonic()
             if timenow >= t2:
                 fut.cancel()
-                raise TotalTimeout(timenow - t0)
+                raise TimeoutError(f"{timenow - t0} seconds")
             await asyncio.sleep(min(self.SLEEP_FUTURE, t2 - timenow))
-        # await asyncio.wait_for(fut, timeout=t0 + total_timeout - monotonic())
+        # await asyncio.wait_for(fut, timeout=t0 + timeout - monotonic())
         # NOTE: `asyncio.wait_for` seems to be blocking for the
         # `timeout` even after result is available.
         return fut.result()
         # This could raise RemoteException.
 
-    def _call_enqueue(self, x, *, enqueue_timeout):
-        uid, fut = self._enqueue_future(x)
+    def _call_wait_for_result(self, fut, *, timeout):
         t0 = fut.data['t0']
-
-        qs = self._input_queues()
-        t1 = t0 + enqueue_timeout
-        for iq, q in enumerate(qs):
-            try:
-                # `enqueue_timeout` is the combined time across input queues,
-                # not time per queue.
-                q.put((uid, x), timeout=max(0, t1 - monotonic()))
-            except Full:
-                fut.cancel()
-                self._uid_to_futures.pop(uid, None)
-                raise EnqueueTimeout(monotonic() - t0, iq)
-
-        fut.data['t1'] = monotonic()
-        return uid, fut
-
-    def _call_wait_for_result(self, uid, fut, *, total_timeout):
-        t0 = fut.data['t0']
-        t2 = t0 + total_timeout
+        t2 = t0 + timeout
         try:
             return fut.result(timeout=max(0, t2 - monotonic()))
             # this may raise RemoteException
         except concurrent.futures.TimeoutError:
             fut.cancel()
-            raise TotalTimeout(monotonic() - t0)
+            raise TimeoutError(f"{monotonic() - t0} seconds")
 
 
 class SequentialServer(MPServer):
@@ -793,34 +731,18 @@ class SequentialServer(MPServer):
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
         self._q_in_out = []
+        self._input_queues_ = None
 
     @overrides
     def _start_servlets(self):
         for k, (servlet, kwargs) in enumerate(self._servlet_configs):
-            qtype = getattr(self.get_mpcontext(), self._queue_type)
-            if self._queue_type == 'ZeroQueue':
-                batch_size = kwargs.get('batch_size', 0)
-                hwm = max(batch_size, 10)
-                if not self._q_in_out:
-                    q_in = qtype(hwm=hwm)
-                    self._q_in_out.append(q_in)
-                else:
-                    q_in = self._q_in_out[-1]
-                if k < len(self._servlet_configs) - 1:
-                    batch_size = self._servlet_configs[k + 1][1].get('batch_size', 0)
-                    hwm = max(batch_size, 10)
-                    q_out = qtype(hwm=hwm)
-                else:
-                    q_out = qtype()
-                self._q_in_out.append(q_out)
+            if not self._q_in_out:
+                q_in = self.get_mpcontext().Unique()
+                self._q_in_out.append(q_in)
             else:
-                if not self._q_in_out:
-                    q_in = qtype()
-                    self._q_in_out.append(q_in)
-                else:
-                    q_in = self._q_in_out[-1]
-                q_out = qtype()
-                self._q_in_out.append(q_out)
+                q_in = self._q_in_out[-1]
+            q_out = self.get_mpcontext().Unique()
+            self._q_in_out.append(q_out)
 
             self._add_servlet(
                 servlet,
@@ -842,6 +764,7 @@ class SequentialServer(MPServer):
         threading.current_thread().name = 'ResultCollector'
         q_out = self._q_in_out[-1]
         futures = self._uid_to_futures
+        q_out = q_out.reader()
 
         while not self._should_stop.is_set():
             try:
@@ -855,10 +778,10 @@ class SequentialServer(MPServer):
                 continue
             try:
                 fut.set_result(y)
-                fut.data['t2'] = monotonic()
+                fut.data['t1'] = monotonic()
             except asyncio.InvalidStateError as e:
                 if fut.cancelled():
-                    # Could have been cancelled due to TotalTimeout.
+                    # Could have been cancelled due to TimeoutError.
                     # logger.debug('Future object is already cancelled')
                     pass
                 else:
@@ -867,7 +790,10 @@ class SequentialServer(MPServer):
 
     @overrides
     def _input_queues(self):
-        return self._q_in_out[:1]
+        if self._input_queues_ is None:
+            q = self._q_in_out[0].writer()
+            self._input_queues_ = [q]
+        return self._input_queues_
 
 
 class EnsembleServer(MPServer):
@@ -881,18 +807,13 @@ class EnsembleServer(MPServer):
         super().__init__(**kwargs)
         self._q_in: List[queues.Queue] = []
         self._q_out: List[queues.Queue] = []
+        self._input_queues_ = None
 
     @overrides
     def _start_servlets(self):
-        qtype = getattr(self.get_mpcontext(), self._queue_type)
-        for servlet, kwargs in self._servlet_configs:
-            if self._queue_type == 'ZeroQueue':
-                batch_size = kwargs.get('batch_size', 0)
-                hwm = max(10, batch_size)
-                q_in = qtype(hwm=hwm)
-            else:
-                q_in = qtype()
-            q_out = qtype()
+        for k, (servlet, kwargs) in enumerate(self._servlet_configs):
+            q_in = self.get_mpcontext().Unique()
+            q_out = self.get_mpcontext().Unique()
             self._q_in.append(q_in)
             self._q_out.append(q_out)
 
@@ -937,12 +858,14 @@ class EnsembleServer(MPServer):
         futures = self._uid_to_futures
         n_results_needed = len(self._q_out)
 
+        qq = [q.reader() for q in self._q_out]
+
         while not self._should_stop.is_set():
-            for idx, q_out in enumerate(self._q_out):
+            for idx, q_out in enumerate(qq):
                 while not q_out.empty():
                     # Get all available results out of this queue.
                     # They are for different requests.
-                    uid, y = q_out.get_nowait()
+                    uid, y = q_out.get(timeout=0)
                     fut = futures.get(uid)
                     if fut is None:
                         # timed-out in `async_call` or `call`.
@@ -964,14 +887,17 @@ class EnsembleServer(MPServer):
                             else:
                                 logger.exception(e)
                                 raise
-                        fut.data['t2'] = monotonic()
+                        fut.data['t1'] = monotonic()
                     # No sleep. Get results out of the queue as quickly as possible.
                     # TODO: check `should_stop`?
             sleep(self.SLEEP_DEQUEUE)
 
     @overrides
     def _input_queues(self):
-        return self._q_in
+        if self._input_queues_ is None:
+            qq = [q.writer() for q in self._q_in]
+            self._input_queues_ = qq
+        return self._input_queues_
 
 
 class SimpleServlet(Servlet):
@@ -990,7 +916,7 @@ class SimpleServer(SequentialServer):
     the specified function.
     '''
 
-    def __init__(self, func: Callable, /, queue_type='BasicQueue', **kwargs):
+    def __init__(self, func: Callable, /, **kwargs):
         '''
         `func`: a function that takes an input value,
             which will be the value provided in calls to the server,
@@ -998,6 +924,6 @@ class SimpleServer(SequentialServer):
             level (can't be defined within a function), and can't be
             a lambda.
         '''
-        super().__init__(queue_type=queue_type)
+        super().__init__()
 
         self.add_servlet(SimpleServlet, func=func, **kwargs)
